IND_INDICES = set(["BSE CG","SENSEX","BSE CD","NIFTY50 PR 1x INV","BSE IT","METAL",
                "OILGAS","NIFTY50 PR 2x LEV","BSEIPO","GREENX","POWER","NIFTY50 TR 1x INV",
                "CARBON","BASMTR","CDGS","NIFTY50 TR 2x LEV","BSEFMC","BSE HC","ALLCAP",
                "NIFTY50 TR 2x LEV","REALTY","SMEIPO","DOL30","NIFTY Mid LIQ 15","LRGCAP",
                "MIDSEL","SMLSEL","NIFTY100 LIQ 15","SNXT50","SNSX50","NIFTY 50","NIFTY Quality 30",
                "NIFTY BANK","NIFTY NEXT 50","DOL100","NIFTY MIDCAP 50","NIFTY 100","NIFTY 200",
                "NIFTY 500","NIFTY FIN SERVICE","NIFTY AUTO","NIFTY FMCG","NIFTY IT","NIFTY COMMODITIES",
                "NIFTY MEDIA","NIFTY METAL","NIFTY PHARMA","NIFTY CONSUMPTION","NIFTY PSU BANK",
                "NIFTY PVT BANK","NIFTY REALTY","NIFTY GROWSECT 15","NIFTY CPSE","NIFTY ENERGY",
                "NIFTY INFRA","NIFTY DIV OPPS 50","NIFTY MNC","NIFTY PSE","NIFTY SERV SECTOR",
                "NIFTY MID100 FREE","DOL200","TECK","BSEPSU","NIFTY SML100 FREE","AUTO","BANKEX",
                "INDIA VIX","NIFTY50 VALUE 20","NIFTY","BANKNIFTY","FINNIFTY"
                ])


from datetime import datetime,date
import pandas as pd

from bandl.helper import get_formated_date,get_date_range,is_ind_index

class NseUrls:
    def __init__(self):
        self.__OPTION_CHAIN_BASE_URL = r"https://www1.nseindia.com/live_market/dynaContent/"\
                                     + r"live_watch/option_chain/optionKeys.jsp?symbol="
        #In EncodedURI
        self.__PARTICIPANT_OI_PRE_URL = r"https://www.nseindia.com/api/reports?archives=%5B%7B%22name"\
                                    + r"%22%3A%22F%26O%20-%20Participant%20wise%20Trading%20Volumes(csv)"\
                                    + r"%22%2C%22type%22%3A%22archives%22%2C%22category%22%3A%22derivatives"\
                                    + r"%22%2C%22section%22%3A%22equity%22%7D%5D&date="
        self.__PARTICIPANT_OI_POST_URL = r"&type=equity&mode=single"

        #Dictionary contains nse date formats for urls
        self.nse_date_formats = {
                                    "opt_chain":'%d%b%Y',
                                    "part_oi":'%d-%b-%Y',
                                    "stock_data":'%d-%m-%Y',
                                    "index_data":'%d-%m-%Y',
                                    "vix_data":'%d-%b-%Y'
                                }

        #browser like header to avoid error 403
        self.header = {'User-Agent': "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36"}
        #participant wise OI
        self.PART_OI_CLM = ['Future Index Long', 'Future Index Short', 'Future Stock Long',
                            'Future Stock Short\t', 'Option Index Call Long',
                            'Option Index Put Long', 'Option Index Call Short',
                            'Option Index Put Short', 'Option Stock Call Long',
                            'Option Stock Put Long', 'Option Stock Call Short',
                            'Option Stock Put Short', 'Total Long Contracts\t',
                            'Total Short Contracts']

        #stock data
        self.__HIST_DATA_PRE_URL = r"https://www.nseindia.com/api/historical/cm/equity?symbol="

        #Historical index data URL
        self.INDEX_URL = r"https://www1.nseindia.com/products/content/equities/indices/historical_index_data.htm"
        self.__HIST_INDEX_DATA_URL = r"https://www1.nseindia.com/products/dynaContent/equities/indices/historicalindices.jsp?indexType="
        self.__HIST_VIX_DATA_URL = r"https://www1.nseindia.com/products/dynaContent/equities/indices/hist_vix_data.jsp?&fromDate="
        self.INDEX_DATA_CLM = ["Date","Open","High","Low","Close","Shares Traded","Turnover( Rs. Cr)"]
        self.VIX_DATA_CLM = ["Date","Open","High","Low","Close","Prev. Close","Change","% Change"]
        self.is_index = lambda symbol : True if "NIFTY" in symbol or "INDIA VIX" == symbol else False

        #new optionchani urls
        self.OC_FIRST_TRY = "https://www.nseindia.com/option-chain"
        self.INDEX_OC_URL = "https://www.nseindia.com/api/option-chain-indices?symbol="
        self.STOCK_OC_URL = "https://www.nseindia.com/api/option-chain-equities?symbol="
        self.BASE_URL = "https://www.nseindia.com"
    def get_option_chain_url(self,symbol,expiry_date=None,dayfirst=False):
        try:
            if expiry_date:
                expiry_date = get_formated_date(expiry_date,self.nse_date_formats["opt_chain"],dayfirst).upper()
                if expiry_date.startswith('0'):
                    expiry_date = expiry_date[1:]
                complete_url =  self.__OPTION_CHAIN_BASE_URL + symbol + "&date=" + expiry_date
                return complete_url
            else:
                return self.__OPTION_CHAIN_BASE_URL + symbol
        except Exception as err:
            raise Exception("Error occurred while getting OC url, Error: ",str(err))

    def get_oc_url(self,symbol):
        if symbol:
            if is_ind_index(symbol):
                return self.INDEX_OC_URL + symbol
            else:
                return self.STOCK_OC_URL + symbol


    def get_participant_oi_url(self,date,dayfirst=False):
        try:
            date = get_formated_date(date,format=self.nse_date_formats["part_oi"],dayfirst=dayfirst)
            url = self.__PARTICIPANT_OI_PRE_URL + date + self.__PARTICIPANT_OI_POST_URL
            return url
        except Exception as err:
            raise Exception("Error occurred while getting participant OI. ", str(err))

    def get_stock_data_url(self,symbol,start,end,series="EQ"):
        try:
            symbol = symbol.upper()
            url = None
            if "INDIA VIX" == symbol:
                start = start.strftime(self.nse_date_formats["vix_data"])
                end = end.strftime(self.nse_date_formats["vix_data"])
                url = self.__HIST_VIX_DATA_URL + start + "&toDate=" + end
            else:
                #time format is same for index data and nifty data
                start = start.strftime(self.nse_date_formats["stock_data"])
                end = end.strftime(self.nse_date_formats["stock_data"])

                if "NIFTY" in symbol:
                    symbol = symbol.replace(' ',"%20").upper()
                    url = self.__HIST_INDEX_DATA_URL + symbol + "&fromDate=" +\
                            start + "&toDate=" + end
                else:
                    url = self.__HIST_DATA_PRE_URL + symbol + r"&series=[%22" + series+\
                        r"%22]&from=" + start + r"&to=" + end + r"&csv=true"
            return url

        except Exception as err:
            raise Exception("Error occurred while getting stock data URL. ", str(err))


import json
import pandas as pd
import datetime
from bandl.request import RequestUrl
from bandl.helper import get_date_range

#default params for url connection
DEFAULT_TIMEOUT = 5 # seconds
MAX_RETRIES = 2

class CoinbaseUrl:
    def __init__(self):
        self.BASE_URL = "https://api.pro.coinbase.com"
        self.HEADER =   {
                        'Content-Type': 'application/json',
                        }
        self.data_columns = ["OpenTime","Low","High","Open","Close","Volume"]

    def __interval_to_garanularity(self,internal):
        time_frame  = pd.Timedelta(internal)
        return time_frame.total_seconds()

    def get_candle_data_url(self,symbol,start,end,interval,dayfirst=False):
        #granularity represents to total seconds
        '''The granularity field must be one of the following values: {60, 300, 900, 3600, 21600, 86400}.
            Otherwise, your request will be rejected. These values correspond to timeslices representing one minute,
            five minutes, fifteen minutes, one hour, six hours, and one day, respectively.'''
        granularity = str(int(self.__interval_to_garanularity(interval)))
        str_start = pd.to_datetime(start,dayfirst=dayfirst).strftime('%Y-%m-%dT%H:%M:%S')
        str_end = pd.to_datetime(end,dayfirst=dayfirst).strftime('%Y-%m-%dT%H:%M:%S')
        url = self.BASE_URL+"/products/"+symbol+"/candles?start="+str_start+"&stop="+\
                str_end+"&granularity="+granularity
        return url

class Coinbase:
    def __init__(self,api_key=None, api_secret=None,timeout=DEFAULT_TIMEOUT,max_retries=MAX_RETRIES):
        #internal initialization
        self.__request = RequestUrl(timeout,max_retries)
        self.urls = CoinbaseUrl()

    def get_data(self,symbol,start=None,end=None,periods=None,interval="1D",dayfirst=False):
        """Coinbase getData API for intraday/Historical data

        :param symbol: crypto symbol
        :type symbol: string
        :param start: start time, defaults to None
        :type start: string optional
        :param end: end time, defaults to None
        :type end: string, optional
        :param periods: No of days, defaults to None
        :type periods: integer, optional
        :param interval: timeframe, defaults to "1D"
        :type interval: string, optional
        :param dayfirst: if date in european style, defaults to False
        :type dayfirst: bool, optional
        :raises ValueError: invalid time
        :raises Exception: for execption
        :return: data requested
        :rtype: pandas.DataFrame
        """
        try:
            s_from,e_till = get_date_range(start=start,end=end,periods=periods,dayfirst=dayfirst)
            if s_from > e_till:
                raise ValueError("End should grater than start.")

            #capitalize symbol
            symbol = symbol.upper()
            data_url = self.urls.get_candle_data_url(symbol,start=s_from,end=e_till,interval=interval)
            res = self.__request.get(data_url,headers=self.urls.HEADER)
            data = json.loads(res.text)
            dfs = pd.DataFrame(data,columns=self.urls.data_columns)
            dfs['OpenTime'] = pd.to_datetime(dfs['OpenTime'], unit='s')
            dfs.set_index("OpenTime",inplace=True)
            return dfs

        except Exception as err:
            raise Exception("Error occurred while fetching data :", str(err))


import requests
import json
import socket

from bandl.request import RequestUrl

#default params for url connection
DEFAULT_TIMEOUT = 5 # seconds
MAX_RETRIES = 2

class AngelUrl:
    def __init__(self,api_key):
        self._BASE_URL = "https://apiconnect.angelbroking.com"
        hostname = socket.gethostname()
        client_local_ip = socket.gethostbyname(hostname)
        client_public_ip = RequestUrl().get('https://api.ipify.org').text

        self.routes =   {
                        "login":"/rest/auth/angelbroking/user/v1/loginByPassword"
                        }

        self.LOGIN_HEADERS = {
                        'Content-Type': 'application/json',
                        'Accept': 'application/json',
                        'X-UserType': 'USER',
                        'X-SourceID': 'WEB',
                        'X-ClientLocalIP': client_local_ip,
                        'X-ClientPublicIP': client_public_ip ,
                        'X-MACAddress': '00:00:00:00:00:00',
                        'X-PrivateKey': api_key
                        }

    def get_url(self,type):
        return self._BASE_URL + self.routes.get(type,None)

class AngelBroking:
    def __init__(self,user_id,password,api_key,timeout=DEFAULT_TIMEOUT,max_retries=MAX_RETRIES):
        #internal initialization
        self.__request = RequestUrl(timeout,max_retries)
        self.urls = AngelUrl(api_key)

        request_body = {
                        "clientcode": user_id,
                        "password": password,
                        }

        #lets login
        res = self.__request.post(self.urls.get_url("login"),
                            data=json.dumps(request_body),
                            headers = self.urls.LOGIN_HEADERS)
        self.__login_res = json.loads(res.text)
        self.__feed_token = self.__login_res.get("data").get("feedToken",None)



import requests
import json
from datetime import datetime,date
import pandas as pd

import sys
sys.path.append(r"F:\stolgo\bandl\lib")
from bandl.helper import get_formated_date,get_date_range,is_ind_index,get_data_resample
from bandl.request import RequestUrl

#default params for url connection
DEFAULT_TIMEOUT = 5 # seconds
MAX_RETRIES = 2

class SamcoUrl:
    def __init__(self):
        #token needs to be set by user application
        self.DATA_HEADER = {
                            'Accept': 'application/json',
                            'x-session-token': None
                            }
        self.LOGIN_HEADERS =  {
                        'Content-Type': 'application/json',
                        'Accept': 'application/json'
                        }
        self.LOGIN_URL = "https://api.stocknote.com/login"

        #historical data url
        self.HIST_INDEX_URL = "https://api.stocknote.com/history/indexCandleData?indexName="
        self.HIST_STK_URL = "https://api.stocknote.com/history/candleData?symbolName="

        #intraday data
        self.INTRA_INDEX_URL = "https://api.stocknote.com/intraday/indexCandleData?indexName="
        self.INTRA_STK_URL = "https://api.stocknote.com/intraday/candleData?symbolName="

        #option chain url
        self.OPTION_CHAIN_URL = "https://api.stocknote.com/option/optionChain"

        self.date_format = {
                            "HIST":"%Y-%m-%d",
                            "INTRA":"%Y-%m-%d %H:%M:%S"
                            }

    def set_session(self,token):
        self.DATA_HEADER = {
                        'Accept': 'application/json',
                        'x-session-token': token
                        }

    def __build_url(self,symbol,start,end,url,date_format):
        symbol = symbol.upper().replace(" ","%20")
        start = start.strftime(date_format).replace(" ","%20").replace(":","%3A")
        end = end.strftime(date_format).replace(" ","%20").replace(":","%3A")
        url_build = url + symbol + "&fromDate=" + start + "&toDate=" + end
        return url_build

    def get_intra_data_url(self,symbol,start,end):
        try:
            url = None
            if is_ind_index(symbol):
                url = self.__build_url(symbol,start,end,self.INTRA_INDEX_URL,self.date_format["INTRA"])
            else:
                url = self.__build_url(symbol,start,end,self.INTRA_STK_URL,self.date_format["INTRA"])
            return url
        except Exception as err:
            raise Exception("Error occurred while getting stock data URL. ", str(err))

    def get_hist_data_url(self,symbol,start,end):
        try:
            url = None
            if is_ind_index(symbol):
                url = self.__build_url(symbol,start,end,self.HIST_INDEX_URL,self.date_format["HIST"])
            else:
                url = self.__build_url(symbol,start,end,self.HIST_STK_URL,self.date_format["HIST"])
            return url
        except Exception as err:
            raise Exception("Error occurred while getting stock data URL. ", str(err))


class Samco:
    def __init__(self,user_id,password,yob,timeout=DEFAULT_TIMEOUT,max_retries=MAX_RETRIES):
        #internal initialization
        self.__request = RequestUrl(timeout,max_retries)
        self.urls = SamcoUrl()

        request_body = {
                        "userId": user_id,
                        "password": password,
                        "yob": yob
                        }

        #lets login
        res = self.__request.post(self.urls.LOGIN_URL,
                            data=json.dumps(request_body),
                            headers = self.urls.LOGIN_HEADERS, verify=False)

        self.login_res = json.loads(res.text)
        #set token
        self.urls.set_session(self.login_res.get("sessionToken"))

    def __get_hist_data(self,symbol,start,end,interval="1D"):
        try:
            url = self.urls.get_hist_data_url(symbol,start,end)
            res = self.__request.get(url,headers=self.urls.DATA_HEADER)
            json_key = "historicalCandleData"
            if is_ind_index(symbol):
                json_key = "indexCandleData"
            hist_data_dict =  json.loads(res.text).get(json_key)
            dfs = pd.json_normalize(hist_data_dict)
            dfs.set_index("date",inplace=True)
            # Converting the index as date
            dfs.index = pd.to_datetime(dfs.index)
            return dfs

        except Exception as err:
            raise Exception("Error occurred for historical data: ",str(err))

    def __get_intra_data(self,symbol,start,end,interval="1M"):
        try:
            url = self.urls.get_intra_data_url(symbol,start,end)
            res = self.__request.get(url,headers=self.urls.DATA_HEADER)
            json_key = "intradayCandleData"
            if is_ind_index(symbol):
                json_key = "indexIntraDayCandleData"
            intra_data =  json.loads(res.text).get(json_key)
            dfs = pd.DataFrame(intra_data)
            dfs.set_index("dateTime",inplace=True)
            # Converting the index as date
            dfs.index = pd.to_datetime(dfs.index)
            return dfs
        except Exception as err:
            raise Exception("Error occurred for historical data: ",str(err))

    def __finetune_df(self,df):
        """drop dataframe out of range time

        :param df: input dataframe
        :type df: pd.DataFrame
        """
        drop_index = (df.between_time("07:00","09:00",include_end=False) + \
                        df.between_time("15:30","17:00",include_start=False)).index
        df.drop(drop_index,inplace=True)

    def get_data(self,symbol,start=None,end=None,periods=None,interval="1D",dayfirst=False):
        """Samco getData API for intraday/Historical data

        :param symbol: stock symbol
        :type symbol: string
        :param start: start time, defaults to None
        :type start: string optional
        :param end: end time, defaults to None
        :type end: string, optional
        :param periods: No of days, defaults to None
        :type periods: integer, optional
        :param interval: timeframe, defaults to "1D"
        :type interval: string, optional
        :param dayfirst: if date in european style, defaults to False
        :type dayfirst: bool, optional
        :raises ValueError: invalid time
        :raises Exception: for execption
        :return: data requested
        :rtype: pandas.DataFrame
        """
        try:
            s_from,e_till = get_date_range(start=start,end=end,periods=periods,dayfirst=dayfirst)
            if s_from > e_till:
                raise ValueError("End should grater than start.")

            #capitalize
            symbol = symbol.upper()
            interval = interval.upper()

            time_frame  = pd.Timedelta(interval)
            #if interval is 1 day, Use historical data API
            day_time_frame = pd.Timedelta("1D")
            min_time_frame = pd.Timedelta("1M")
            if time_frame >= day_time_frame:
                dfs = self.__get_hist_data(symbol,s_from,e_till)
                dfs = dfs.apply(pd.to_numeric)
                if time_frame != day_time_frame:
                    dfs = get_data_resample(dfs,interval)
            else:
                dfs = self.__get_intra_data(symbol,s_from,e_till)
                dfs = dfs.apply(pd.to_numeric)
                if time_frame != min_time_frame:
                    dfs = get_data_resample(dfs,interval)

            if not dfs.empty:
                return dfs

        except Exception as err:
            raise Exception("Error occurred while fetching data :", str(err))

    def get_optionchain(self,symbol):
        params={'searchSymbolName': symbol}
        res = self.__request.get(self.urls.OPTION_CHAIN_URL,headers=self.urls.DATA_HEADER,params = params)
        return res.json()


import json
import pandas as pd
import datetime

from bandl.request import RequestUrl
from bandl.helper import get_date_range

#default params for url connection
DEFAULT_TIMEOUT = 5 # seconds
MAX_RETRIES = 2

class BinanceUrl:
    def __init__(self):
        self.BASE_URL = "https://api.binance.com/"
        self.BACKUP_URL = "https://api3.binance.com/"
        self.HEADER =   {
                        'Content-Type': 'application/json',
                        }
        self.data_columns = ["OpenTime","Open","High","Low","Close","Volume","CloseTime",\
                            "QuoteAssetVolume","NumberOfTrades","TakerBuyBaseAssetVolume",\
                            "TakerBuyQuoteAssetVolume","Ignore"]
        self.TICKER_URL= "https://www.binance.com/api/v1/ticker/allPrices"

    def get_candle_data_url(self,symbol,start,end,interval,use_backup=False):
        url = self.BASE_URL
        if use_backup:
            url = self.BACKUP_URL
        return url + "api/v3/klines?symbol="+ symbol + "&startTime="+ start + "&endTime=" + end + "&interval=" + interval


class Binance:
    def __init__(self,api_key=None, api_secret=None,timeout=DEFAULT_TIMEOUT,max_retries=MAX_RETRIES):
        #internal initialization
        self.__request = RequestUrl(timeout,max_retries)
        self.urls = BinanceUrl()

    def get_tickers(self,keyword=None):
        """Get all crypto tickers from binance

        :param keyword: Any keyword to match, for ex. "BTC" will return all BTC pair, defaults to "None"
        :type keyword: str, optional
        :raises Exception: related to network/API
        :return: list of all tickers form binance
        :rtype: list
        """
        try:
            res = self.__request.get(self.urls.TICKER_URL,headers=self.urls.HEADER)
            tickers = json.loads(res.text)
            all_tickers = [each_symbol.get("symbol") for each_symbol in tickers]
            if keyword:
                keyword = keyword.upper()
                all_tickers = [symbol for symbol in all_tickers if keyword in symbol]
            return all_tickers
        except Exception as err:
            raise Exception("Error occurred while getting tickers :", str(err))

    def get_data(self,symbol,start=None,end=None,periods=None,interval="1D",dayfirst=False):
        """Binance getData API for intraday/Historical data

        :param symbol: crypto symbol
        :type symbol: string
        :param start: start time, defaults to None
        :type start: string optional
        :param end: end time, defaults to None
        :type end: string, optional
        :param periods: No of days, defaults to None
        :type periods: integer, optional
        :param interval: timeframe, defaults to "1D"
        :type interval: string, optional
        :param dayfirst: if date in european style, defaults to False
        :type dayfirst: bool, optional
        :raises ValueError: invalid time
        :raises Exception: for execption
        :return: data requested
        :rtype: pandas.DataFrame
        """
        try:
            s_from,e_till = get_date_range(start=start,end=end,periods=periods,dayfirst=dayfirst)
            if s_from > e_till:
                raise ValueError("End should grater than start.")

            #capitalize
            symbol = symbol.upper()
            interval = interval.lower()

            s_from_milli_sec = str(int(s_from.timestamp() * 1000))
            e_till_milli_sec = str(int(e_till.timestamp() * 1000))

            data_url = self.urls.get_candle_data_url(symbol,start=s_from_milli_sec,end=e_till_milli_sec,interval=interval)
            try:
                res = self.__request.get(data_url,headers=self.urls.HEADER)
            except:
                data_url = self.urls.get_candle_data_url(symbol,start=s_from_milli_sec,end=e_till_milli_sec,interval=interval,use_backup=True)
                res = self.__request.get(data_url,headers=self.urls.HEADER)

            data = json.loads(res.text)
            dfs = pd.DataFrame(data,columns=self.urls.data_columns)
            dfs['OpenTime'] = pd.to_datetime(dfs['OpenTime'], unit='ms')
            dfs['CloseTime'] = pd.to_datetime(dfs['CloseTime'], unit='ms')
            dfs.set_index("OpenTime",inplace=True)

            return dfs

        except Exception as err:
            raise Exception("Error occurred while fetching data :", str(err))


from datetime import datetime,timedelta
import pandas as pd
import io
import time

from bandl.helper import get_formated_date,get_date_range,is_ind_index
from bandl.request import RequestUrl
#default params for url connection
DEFAULT_TIMEOUT = 5 # seconds
MAX_RETRIES = 3


class Yfinance:
    def __init__(self,timeout=DEFAULT_TIMEOUT,max_retries=MAX_RETRIES):
        self.__yfinance_base_url = r"https://query1.finance.yahoo.com/v7/finance/download/"
        self.__yfinance_suffix_url = r"&interval=1d&events=history&includeAdjustedClose=true"
        self.__headers =  {'User-Agent': "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36"}
        #create request
        self.__request = RequestUrl(timeout,max_retries)
        #ticker symbol map to yahoo finance:
        self.INDEX_MAP = {"SENSEX":"%5EBSESN","NIFTY 50":"%5ENSEI","NIFTY":"%5ENSEI",\
                           "NIFTY BANK":"%5ENSEBANK","BANKNIFTY":"%5ENSEBANK"}

    def __get_complete_url(self,symbol,start,end,is_indian=True):
        period1=int(time.mktime(start.timetuple()))
        period2=int(time.mktime(end.timetuple()))
        symbol = symbol.upper()
        if is_indian and is_ind_index(symbol):
            symbol  = self.INDEX_MAP.get(symbol)
            if not symbol:
                raise Exception("Data not available for this symbol")
        elif(is_indian):
            symbol += ".NS"

        complete_csv_url = self.__yfinance_base_url + symbol +'?period1='+str(period1)+'&period2='+str(period2)+self.__yfinance_suffix_url
        return complete_csv_url


    def get_data(self,symbol,is_indian=True,start=None,end=None,periods=None,dayfirst=False):
        """get_data API to fetch data from nasdaq

        :param symbol: stock symbol
        :type symbol: string
        :param start: start date, defaults to None
        :type start: string, optional
        :param end: end date, defaults to None
        :type end: string, optional
        :param is_indian: False if stock is not from indian market , defaults to True
        :type is_indian: bool, optional
        :param periods: number of days, defaults to None
        :type periods: integer, optional
        :param dayfirst: True if date format is european style DD/MM/YYYY, defaults to False
        :type dayfirst: bool, optional
        :raises ValueError: for invalid inputs
        :raises Exception: incase if no data found
        :return: stock data
        :rtype: pandas.DataFrame
        """
        #Step1: get the date range
        s_from,e_till = get_date_range(start=start,end=end,periods=periods,dayfirst=dayfirst)
        url = self.__get_complete_url(symbol,s_from,e_till,is_indian)
        response=self.__request.get(url,headers=self.__headers)
        dfs = pd.read_csv(io.StringIO(response.content.decode('utf-8')))
        dfs.set_index("Date",inplace=True)
        #convert to  datetime
        dfs.index = pd.to_datetime(dfs.index)
        dfs = self.__get_data_adjusted(dfs,symbol,start=start,end=end,periods=periods)
        if not dfs.empty:
            return dfs

    def __join_dfs(self,join,joiner):
        """will append joiner to join for oi_dfs

        :param join: df which will be appended
        :type join: pandas.DataFrame
        :param joiner: df which we want to append
        :type joiner: pandas.DataFrame
        :return: merged data frame
        :rtype: pandas.DataFrame
        """
        return join.append(joiner)

    def __get_data_adjusted(self,dfs,symbol,start=None,end=None,periods=None):
        if periods and (dfs.shape[0] < periods):
            new_periods = periods - dfs.shape[0]
            try:
                s_from = e_till = None
                #if only start, find till today
                if start and (not end):
                    s_from = dfs.index[0] + timedelta(1)
                    e_till = None
                #if not start, can go to past
                elif((end and (not start)) or periods):
                    s_from = None
                    e_till = dfs.index[0] - timedelta(1)
            except IndexError as err:
                raise Exception("yahoo finace Access error.")
            except Exception as exc:
                raise Exception("yahoo finace data error: ",str(exc))
            try:
                dfs_new = self.get_data(symbol,start = s_from,end = e_till,periods = new_periods)
                dfs = self.__join_dfs(dfs,dfs_new).sort_index(ascending=False)
            except Exception as exc:
                #Small part of data may not be available
                pass
        return dfs


import configparser
import json
import sys
from bandl.exception import InvalidFileError
from bandl.request import RequestUrl

#default params for url connection
DEFAULT_TIMEOUT = 5 # seconds
MAX_RETRIES = 2

from Crypto.Cipher import AES
import base64
from pbkdf2 import PBKDF2


class EncryptionClient:
    # Credit : https://github.com/5paisa/py5paisa/blob/master/py5paisa/auth.py
    def __init__(self,enc_key):
        self.iv = bytes([83, 71, 26, 58, 54, 35, 22, 11,
                         83, 71, 26, 58, 54, 35, 22, 11])
        self.enc_key = enc_key

    def _pad_and_convert_to_bytes(self, text):
        return bytes(text+chr(16-len(text) % 16)*(16-len(text) % 16), encoding="utf-8")

    def encrypt(self, text):
        padded_text = self._pad_and_convert_to_bytes(text)
        key_gen = PBKDF2(self.enc_key, self.iv)

        aesiv = key_gen.read(16)
        aeskey = key_gen.read(32)
        cipher = AES.new(aeskey, AES.MODE_CBC, aesiv)

        return str(base64.b64encode(cipher.encrypt(padded_text)), encoding="utf-8")



class FivePaisaURL:
    def __init__(self,email,password,dob,apikeys_filepath):
        self.LOGIN_URL=r"https://Openapi.5paisa.com/VendorsAPI/Service1.svc/V2/LoginRequestMobileNewbyEmail"
        self.LOGIN_HEADERS =  {'Content-Type': 'application/json'}
        config = configparser.ConfigParser()
        try:
            config.read(apikeys_filepath)
            self.__app_name = config["KEYS"]["APP_NAME"]
            self.__app_src =    config["KEYS"]["APP_SOURCE"]
            self.__used_id =    config["KEYS"]["USER_ID"]
            self.__password =   config["KEYS"]["PASSWORD"]
            self.__user_key =   config["KEYS"]["USER_KEY"]
            self.__encryption_key =config["KEYS"]["ENCRYPTION_KEY"]

        except Exception as err:
            raise InvalidFileError("API Keys configuration file is invalid")

        encryption_client = EncryptionClient(self.__encryption_key)
        self.LOGIN_PAYLOAD = {"head": {
            "appName": self.__app_name,
            "appVer": "1.0",
            "key": self.__user_key ,
            "osName": "WEB",
            "requestCode": "5PLoginV2",
            "userId": self.__used_id,
            "password": self.__password
            },
            "body":
            {
            "Email_id": encryption_client.encrypt(email),
            "Password": encryption_client.encrypt(password),
            "LocalIP": "",
            "PublicIP": "",
            "HDSerailNumber": "",
            "MACAddress": "",
            "MachineID": "039377",
            "VersionNo": "1.7",
            "RequestNo": "1",
            "My2PIN": encryption_client.encrypt(dob),
            "ConnectionType": "1"
            }
        }

class FivePaisa():
    def __init__(self,email,password,dob,apikeys_filepath,timeout=DEFAULT_TIMEOUT,max_retries=MAX_RETRIES):
        self.urls = FivePaisaURL(email,password,dob,apikeys_filepath)
        #internal initialization
        self.__request = RequestUrl(timeout,max_retries)
        #lets login
        res = self.__request.post(self.urls.LOGIN_URL,
                            data=json.dumps(self.urls.LOGIN_PAYLOAD),
                            headers = self.urls.LOGIN_HEADERS, verify=False)

        self.login_res = json.loads(res.text)



from setuptools import setup,find_packages

with open("README.md", 'r') as f:
    README = f.read()

setup(
   name='bandl',
   version='0.1.0',
   description='Utilities for the analysis of financial data',
   license="MIT",
   long_description=README,
   long_description_content_type='text/markdown',
   author='stolgo Developers',
   author_email='stockalgos@gmail.com',
   project_urls={
          "Organization":"http://www.bandl.io",
          "Source":"https://github.com/stockalgo/bandl",
          "Tracker":"https://github.com/stockalgo/bandl/issues"
          },
   packages=find_packages('lib'),
   package_dir = {'':'lib'},
   include_package_data=True,
   install_requires=[
            'requests',
            'pandas',
            'datetime',
            'openpyxl',
            'futures',
            'beautifulsoup4',
            'lxml'],
    classifiers=[
      'Development Status :: 5 - Production/Stable',
      'Intended Audience :: Developers',
      'Topic :: Software Development :: Build Tools',
      'License :: OSI Approved :: MIT License',
      'Programming Language :: Python :: 3',
      'Programming Language :: Python :: 3.5',
      'Programming Language :: Python :: 3.6',
      'Programming Language :: Python :: 3.7',
      'Programming Language :: Python :: 3.8',
        ],
    download_url = "https://github.com/stockalgo/bandl/archive/v0.1.0.tar.gz",
    keywords = ["python","yahoo-finance-api","coinbase-api","nse","binance-api","coinbase-pro",\
                "5paisa","nasdaq-crawler","yfinance","5paisa-trading-apis","nasdaq-python-api",\
                "yfinance-api","nasqad","samco","nse-python-api","optionchain","angelbroking-apis",\
                  "binance-python-api","coinbase-python-api","samco-python-api"]
)


class BadDataError(Exception):
    def __init__(self, message):
        super(BadDataError, self).__init__(message)

class InvalidFileError(Exception):
    def __init__(self,message):
        super(InvalidFileError, self).__init__(message)